function Sts = simulate_server(max_time)
   st = initial_state();
   Sts = [st];
   while !stop(Sts(end), max_time)
	   st = update(st);
	   Sts = [Sts, st];
   end
end

function st = initial_state()
   st.time_stamp = 0;
   st.server_busy = false;             % the server is idele
   st.end_service = +inf;              % not applicable
   st.next_arrival = next_arrival();
   st.n_accept = 0;
   st.n_reject = 0;  
% printf("starting\n"); % trace – can be ommited
end

function t_next = next_arrival()
% an exponential distribution with lambda = 0.2
% i.e. mean time between arrivals of 1/lambda = 5	
   lambda = 0.2; r = rand();	
   t_next = -log(1-r)/lambda;
end

function t_busy = service_time()
% a uniform distribution between 2 and 8
   t_busy = 2 + 6*rand(); % average time = (2+8)/2 = 5
end

function b = stop(st, max_time)
   b = (st.time_stamp >= max_time);
end

function st1 = update(st0)
   if       st0.server_busy && ...
            st0.next_arrival < st0.end_service
      st1 = reject_event(st0);     % a reject event
   elseif   st0.server_busy && ...
            st0.end_service <= st0.next_arrival   
      st1 = serviced_event(st0);    % a serviced event  
   elseif   !st0.server_busy
       st1 = arrival_event(st0);   % a dequeued event
   else
      printf("error - unclear transition in state:\n")
      st1 = st0
      st1.time_stamp = + inf;
   end;
end

function st1 = reject_event(st0)
   t = st0.next_arrival;
   st1.time_stamp = t;
   st1.server_busy = st0.server_busy;       % server is kept busy
   st1.end_service = st0.end_service;	      % end service time is kept
   st1.next_arrival = t + next_arrival();  % next arrival time is computed
	st1.n_accept = st0.n_accept;  	         % maintain accepted requests
   st1.n_reject = st0.n_reject + 1;  	      % increase rejected requests
   printf("rejected at %6.2f: busy until %6.2f; next arrival \
at %6.2f\n", t, st1.end_service, st1.next_arrival);
end 
       
function st1 = arrival_event(st0)
   t = st0.next_arrival;
   st1.time_stamp = t;
   st1.server_busy = true;        	      % server becomes busy
   st1.end_service = t + service_time();	% new end service time 
	st1.next_arrival = t + next_arrival();	% next arrival time is kept
 	st1.n_accept = st0.n_accept + 1;	      % increase accepted requests
   st1.n_reject = st0.n_reject;     	   % maintain rejected requests
   printf("arriving at %6.2f: busy until %6.2f; next arrival \
at %6.2f\n", t, st1.end_service, st1.next_arrival);
end

function st1 = serviced_event(st0)
   t = st0.end_service;
   st1.time_stamp = t;
	st1.server_busy = false;     	         % server becomes not busy
   st1.end_service = +inf;     	         % serviced time is not applicable
   st1.next_arrival = st0.next_arrival;	% maintain next arrival time
   st1.n_accept = st0.n_accept;  	      % maintain accepted requests
   st1.n_reject = st0.n_reject;           % maintain rejected requests
   printf("serviced at %6.2f: busy until %6.2f; next arrival \
at %6.2f\n", t, st1.end_service, st1.next_arrival);
end



